/***************************************************************************
 *   Copyright (C) 2007 by Lisandro Damián Nicanor Pérez Meyer             *
 *   perezmeyer@gmail.com                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <QPointF>
#include <QDebug>
#include "cube.h"
#include "scanlinetable.h"

Cube::Cube()
{
	rightTopBack = Point4F(0.25, -0.25, -0.25);
	rightBottomBack = Point4F(0.25, 0.25, -0.25);
	leftBottomBack = Point4F(-0.25, 0.25, -0.25);
	leftTopBack = Point4F(-0.25, -0.25, -0.25);

	rightTopFront = Point4F(0.25, -0.25, 0.25);
	rightBottomFront = Point4F(0.25, 0.25, 0.25);
	leftBottomFront = Point4F(-0.25, 0.25, 0.25);
	leftTopFront = Point4F(-0.25, -0.25, 0.25);
}

void Cube::draw(const Matrix4X4 & incomingMatrix, Matrix * zbuffer,
                bool perspective, QImage * image)
{
	Matrix4X4 temp = getTransformationMatrix();
	Matrix4X4 matrix = incomingMatrix.multiplyToTheRight(temp);

	ScanLineTable * table = new ScanLineTable(zbuffer->getRows());

	// Draw the square
	// Back
	drawZBufferLine(matrix.multiplyPoint(rightTopBack),
	                matrix.multiplyPoint(rightBottomBack),
	                QColor("blue"), QColor("blue"), *table,
	                zbuffer, perspective, image);
	drawZBufferLine(matrix.multiplyPoint(rightBottomBack),
	                matrix.multiplyPoint(leftBottomBack),
	                QColor("blue"), QColor("blue"), *table,
                 zbuffer, perspective, image);
	drawZBufferLine(matrix.multiplyPoint(leftBottomBack),
	                matrix.multiplyPoint(leftTopBack),
	                QColor("blue"), QColor("blue"), *table,
                 zbuffer, perspective, image);
	drawZBufferLine(matrix.multiplyPoint(leftTopBack),
	                matrix.multiplyPoint(rightTopBack),
	                QColor("blue"), QColor("blue"), *table,
                 zbuffer, perspective, image);

	drawScanLineTable(*table, zbuffer, perspective, image);
	delete table;

	table = new ScanLineTable(zbuffer->getRows());

	// Front
	drawZBufferLine(matrix.multiplyPoint(rightTopFront),
	                matrix.multiplyPoint(rightBottomFront),
	                QColor("green"), QColor("blue"), *table,
                 zbuffer, perspective, image);
	drawZBufferLine(matrix.multiplyPoint(rightBottomFront),
	                matrix.multiplyPoint(leftBottomFront),
	                QColor("green"), QColor("red"), *table,
                 zbuffer, perspective, image);
	drawZBufferLine(matrix.multiplyPoint(leftBottomFront),
	                matrix.multiplyPoint(leftTopFront),
	                QColor("green"), QColor("green"), *table,
                 zbuffer, perspective, image);
	drawZBufferLine(matrix.multiplyPoint(leftTopFront),
	                matrix.multiplyPoint(rightTopFront),
	                QColor("green"), QColor("green"), *table,
                 zbuffer, perspective, image);

	drawScanLineTable(*table, zbuffer, perspective, image);
	delete table;

	table = new ScanLineTable(zbuffer->getRows());

	// Left side
	drawZBufferLine(matrix.multiplyPoint(leftTopFront),
	                matrix.multiplyPoint(leftTopBack),
	                QColor("yellow"), QColor("yellow"), *table,
                 zbuffer, perspective, image);
	drawZBufferLine(matrix.multiplyPoint(leftBottomFront),
	                matrix.multiplyPoint(leftBottomBack),
	                QColor("yellow"), QColor("yellow"), *table,
                 zbuffer, perspective, image);
	drawZBufferLine(matrix.multiplyPoint(leftTopFront),
	                matrix.multiplyPoint(leftBottomFront),
	                QColor("yellow"), QColor("yellow"), *table,
                 zbuffer, perspective, image);
	drawZBufferLine(matrix.multiplyPoint(leftTopBack),
	                matrix.multiplyPoint(leftBottomBack),
	                QColor("yellow"), QColor("yellow"), *table,
                 zbuffer, perspective, image);

	drawScanLineTable(*table, zbuffer, perspective, image);
	delete table;

	table = new ScanLineTable(zbuffer->getRows());

	// Right side
	drawZBufferLine(matrix.multiplyPoint(rightTopFront),
	                matrix.multiplyPoint(rightTopBack),
	                QColor("white"), QColor("white"), *table,
                 zbuffer, perspective, image);
	drawZBufferLine(matrix.multiplyPoint(rightBottomFront),
	                matrix.multiplyPoint(rightBottomBack),
	                QColor("white"), QColor("white"), *table,
                 zbuffer, perspective, image);
	drawZBufferLine(matrix.multiplyPoint(rightTopFront),
	                matrix.multiplyPoint(rightBottomFront),
	                QColor("white"), QColor("white"), *table,
                 zbuffer, perspective, image);
	drawZBufferLine(matrix.multiplyPoint(rightTopBack),
	                matrix.multiplyPoint(rightBottomBack),
	                QColor("white"), QColor("white"), *table,
                 zbuffer, perspective, image);


	drawScanLineTable(*table, zbuffer, perspective, image);
	delete table;

	table = new ScanLineTable(zbuffer->getRows());

	// Top
	drawZBufferLine(matrix.multiplyPoint(rightTopFront),
	                matrix.multiplyPoint(rightTopBack),
	                QColor("red"), QColor("red"), *table,
                 zbuffer, perspective, image);
	drawZBufferLine(matrix.multiplyPoint(rightTopBack),
	                matrix.multiplyPoint(leftTopBack),
	                QColor("red"), QColor("red"), *table,
                 zbuffer, perspective, image);
	drawZBufferLine(matrix.multiplyPoint(leftTopBack),
	                matrix.multiplyPoint(leftTopFront),
	                QColor("red"), QColor("red"), *table,
                 zbuffer, perspective, image);
	drawZBufferLine(matrix.multiplyPoint(leftTopFront),
	                matrix.multiplyPoint(rightTopFront),
	                QColor("red"), QColor("red"), *table,
                 zbuffer, perspective, image);

	drawScanLineTable(*table, zbuffer, perspective, image);
	delete table;

	table = new ScanLineTable(zbuffer->getRows());

	// Bottom
	drawZBufferLine(matrix.multiplyPoint(rightBottomFront),
	                matrix.multiplyPoint(rightBottomBack),
	                QColor("magenta"), QColor("magenta"), *table,
                 zbuffer, perspective, image);
	drawZBufferLine(matrix.multiplyPoint(rightBottomBack),
	                matrix.multiplyPoint(leftBottomBack),
	                QColor("magenta"), QColor("magenta"), *table,
                 zbuffer, perspective, image);
	drawZBufferLine(matrix.multiplyPoint(leftBottomBack),
	                matrix.multiplyPoint(leftBottomFront),
	                QColor("magenta"), QColor("magenta"), *table,
                 zbuffer, perspective, image);
	drawZBufferLine(matrix.multiplyPoint(leftBottomFront),
	                matrix.multiplyPoint(rightBottomFront),
	                QColor("magenta"), QColor("magenta"), *table,
                 zbuffer, perspective, image);

	drawScanLineTable(*table, zbuffer, perspective, image);
	delete table;
}




