/***************************************************************************
 *   Copyright (C) 2007 by Lisandro Damián Nicanor Pérez Meyer             *
 *   perezmeyer@gmail.com                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <QColor>
#include <QImage>
#include <QDebug>
#include <math.h>
#include "point4f.h"
#include "graphicprimitive.h"
#include "scanlinetable.h"

GraphicPrimitive::GraphicPrimitive()
{
	xDisplacement = 0.0;
	yDisplacement = 0.0;
	zDisplacement = 0.0;
	angleOfRotationX = 0.0;
	angleOfRotationY = 0.0;
	angleOfRotationZ = 0.0;
	xScale = 1.0;
	yScale = 1.0;
	zScale = 1.0;
}

double GraphicPrimitive::getXDisplacement()
{
	return xDisplacement;
}

double GraphicPrimitive::getYDisplacement()
{
	return yDisplacement;
}

double GraphicPrimitive::getZDisplacement()
{
	return zDisplacement;
}

Matrix4X4 GraphicPrimitive::getDisplacementMatrix()
{
	Matrix4X4 matrix;

	matrix.setValue(0, 0, 1.0);
	matrix.setValue(0, 1, 0.0);
	matrix.setValue(0, 2, 0.0);
	matrix.setValue(0, 3, xDisplacement);

	matrix.setValue(1, 0, 0.0);
	matrix.setValue(1, 1, 1.0);
	matrix.setValue(1, 2, 0.0);
	matrix.setValue(1, 3, yDisplacement);

	matrix.setValue(2, 0, 0.0);
	matrix.setValue(2, 1, 0.0);
	matrix.setValue(2, 2, 1.0);
	matrix.setValue(2, 3, zDisplacement);

	matrix.setValue(3, 0, 0.0);
	matrix.setValue(3, 1, 0.0);
	matrix.setValue(3, 2, 0.0);
	matrix.setValue(3, 3, 1.0);

	return matrix;
}

double GraphicPrimitive::getAngleOfRotationX()
{
	return angleOfRotationX;
}

double GraphicPrimitive::getAngleOfRotationY()
{
	return angleOfRotationY;
}

double GraphicPrimitive::getAngleOfRotationZ()
{
	return angleOfRotationZ;
}

Matrix4X4 GraphicPrimitive::getRotationMatrix()
{
	Matrix4X4 matrix;

	matrix.setValue(0, 0, cos(angleOfRotationY) * cos(angleOfRotationZ));
	matrix.setValue(0, 1, cos(angleOfRotationY) * -sin(angleOfRotationZ));
	matrix.setValue(0, 2, -sin(angleOfRotationY));
	matrix.setValue(0, 3, 0.0);

	matrix.setValue(1, 0, -sin(angleOfRotationX) * sin(angleOfRotationY) *
	                cos(angleOfRotationZ) + cos(angleOfRotationX) *
	                sin(angleOfRotationZ));
	matrix.setValue(1, 1, sin(angleOfRotationX) * sin(angleOfRotationY) *
	                sin(angleOfRotationZ) + cos(angleOfRotationX) *
	                cos(angleOfRotationZ));
	matrix.setValue(1, 2, -sin(angleOfRotationX) * cos(angleOfRotationY));
	matrix.setValue(1, 3, 0.0);

	matrix.setValue(2, 0, cos(angleOfRotationX) * sin(angleOfRotationY) *
	                cos(angleOfRotationZ) + sin(angleOfRotationX) *
	                sin(angleOfRotationZ));
	matrix.setValue(2, 1, cos(angleOfRotationX) * sin(angleOfRotationY) *
	                -sin(angleOfRotationZ) + sin(angleOfRotationX) *
	                cos(angleOfRotationZ));
	matrix.setValue(2, 2, cos(angleOfRotationX) * cos(angleOfRotationY));
	matrix.setValue(2, 3, 0.0);

	matrix.setValue(3, 0, 0.0);
	matrix.setValue(3, 1, 0.0);
	matrix.setValue(3, 2, 0.0);
	matrix.setValue(3, 3, 1.0);

	return matrix;
}

double GraphicPrimitive::getXScale()
{
	return xScale;
}

double GraphicPrimitive::getYScale()
{
	return yScale;
}

double GraphicPrimitive::getZScale()
{
	return zScale;
}

Matrix4X4 GraphicPrimitive::getScaleMatrix()
{
	Matrix4X4 matrix;

	matrix.setValue(0, 0, xScale);
	matrix.setValue(0, 1, 0.0);
	matrix.setValue(0, 2, 0.0);
	matrix.setValue(0, 3, 0.0);

	matrix.setValue(1, 0, 0.0);
	matrix.setValue(1, 1, yScale);
	matrix.setValue(1, 2, 0.0);
	matrix.setValue(1, 3, 0.0);

	matrix.setValue(2, 0, 0.0);
	matrix.setValue(2, 1, 0.0);
	matrix.setValue(2, 2, zScale);
	matrix.setValue(2, 3, 0.0);

	matrix.setValue(3, 0, 0.0);
	matrix.setValue(3, 1, 0.0);
	matrix.setValue(3, 2, 0.0);
	matrix.setValue(3, 3, 1.0);

	return matrix;
}

Matrix4X4 GraphicPrimitive::getTransformationMatrix()
{
	Matrix4X4 scale = getScaleMatrix();
	Matrix4X4 rotation = getRotationMatrix();
	Matrix4X4 displacement = getDisplacementMatrix();

	Matrix4X4 temp = rotation.multiplyToTheRight(scale);
	Matrix4X4 matrix = displacement.multiplyToTheRight(temp);

	return matrix;
}

void GraphicPrimitive::setXDisplacement(double displacement)
{
	xDisplacement = displacement;
}

void GraphicPrimitive::setYDisplacement(double displacement)
{
	yDisplacement = displacement;
}

void GraphicPrimitive::setZDisplacement(double displacement)
{
	zDisplacement = displacement;
}

void GraphicPrimitive::setAngleOfRotationX(double theta)
{
	angleOfRotationX = theta;
}

void GraphicPrimitive::setAngleOfRotationY(double theta)
{
	angleOfRotationY = theta;
}

void GraphicPrimitive::setAngleOfRotationZ(double theta)
{
	angleOfRotationZ = theta;
}

void GraphicPrimitive::setXScale(double scale)
{
	xScale = scale;
}

void GraphicPrimitive::setYScale(double scale)
{
	yScale = scale;
}

void GraphicPrimitive::setZScale(double scale)
{
	zScale = scale;
}

/**
	Interpolates a line and adds the data generated to the scabn line table.
*/
void GraphicPrimitive::drawZBufferLine(Point4F begin, Point4F end,
                                       const QColor firstColor, const QColor secondColor,
                                       ScanLineTable & table, Matrix * zbuffer,
                                       bool perspective, QImage * image)
{
	QColor tempColor("white");
	double zValue(0.0);

	/// FIXME or EXPLAINME
	/**
	  This is a fix proposed by my teacher. I don't know why it works.
	*/
	if(perspective)
	{
		begin.rx() /= begin.z() + 1;
		begin.ry() /= begin.z() + 1;
		end.rx() /= end.z() + 1;
		end.ry() /= end.z() + 1;
	}

	/*
		First of all, we will pass the coordinates to viewport coordinates.

		FIXME
		We will assume a viewport defined by [-1, 1] x [-1, 1].
		Note that drawScanLineTable also has this limitations
	*/
	double minimum = qMin(image->width(), image->height());

	double xVBegin = (begin.x() + 1.0) / 2.0 * minimum +
	                 ((image->width() - minimum) / 2.0);
	double yVBegin = (begin.y() + 1.0) / 2.0 * minimum +
	                 ((image->height() - minimum) / 2.0);
	double xVEnd = (end.x() + 1.0) / 2.0 * minimum +
	               ((image->width() - minimum) / 2.0);
	double yVEnd = (end.y() + 1.0) / 2.0 * minimum +
	               ((image->height() - minimum) / 2.0);

	int beginY = floor(yVBegin);
	int endY = floor(yVEnd);
	double x = 0.0;
	int floorX = 0;

	bool negativeDelta = false;
	if((yVEnd - yVBegin) < 0)
		negativeDelta = true;

	int currentY = beginY;

	// Generate the line data
	while(currentY != endY)
	{
		// Calculate x
		x = (currentY - yVBegin) / (yVEnd - yVBegin);
		x *= (xVEnd - xVBegin);
		x += xVBegin;
		floorX = floor(x);

		// Calculate the color
		tempColor = interpolateColors(firstColor, secondColor,
			                            beginY, endY, currentY);
		// Calculate the Z value
		zValue = interpolateZ(begin, end, beginY, endY, currentY);

		/*
			Check wether currentY is between the bounds and if correct, add the info.
		*/
		if((currentY >= 0) && (currentY < image->height()))
		{
			// Add the data to the scan line table
			table.addItem(floorX, currentY, zValue, tempColor);
		}

		if(negativeDelta)
			currentY--;
		else
			currentY++;
	}
}


QColor GraphicPrimitive::interpolateColors(const QColor firstColor,
                                           const QColor secondColor,
                                           const int begin, const int end,
                                           const int position)
{
	if(begin == end)
		return QColor(0, 0, 0);

	double r(0), g(0), b(0);

	r = (double)(secondColor.red() - firstColor.red())/(double)(end - begin);
	r *= (position - begin);
	r += firstColor.red();

	g = (double)(secondColor.green() - firstColor.green())/(double)(end - begin);
	g *= (position - begin);
	g += firstColor.green();

	b = (double)(secondColor.blue() - firstColor.blue())/(double)(end - begin);
	b *= (position - begin);
	b += firstColor.blue();

	return QColor((int)r, (int)g, (int)b);
}

double GraphicPrimitive::interpolateZ(const Point4F & firstPoint,
                                      const Point4F & secondPoint,
                                      const int begin, const int end,
                                      const int position)
{
	if(begin == end)
		return 0.0;

	double z = 0.0;
	z = (secondPoint.z() - firstPoint.z()) / (double) (end - begin);
	z *= (double)(position - begin);
	z += firstPoint.z();

	return z;
}

void GraphicPrimitive::drawScanLineTable(ScanLineTable & table,
                                         Matrix * zbuffer, bool perspective,
                                         QImage * image)
{
	double minX(0.0), maxX(0.0), minZ(0.0), maxZ(0.0);
	QColor minColor("black"), maxColor("black"), tempColor("black");
	double z = 0.0;

	for(int row=0; row < table.getSize(); row++)
	{
		table.getXMinMax(row, minX, minZ, minColor, maxX, maxZ, maxColor);

		if(minX != maxX)
			for(int column = minX; column <= maxX; column++)
			{
				tempColor = interpolateColors(minColor, maxColor, minX, maxX, column);
				z = interpolateZ(Point4F(minX, row, minZ, 1.0),
				                 Point4F(maxX, row, maxZ, 1.0),
				                 minX, maxX, column);

				if((column >=0) && (column < image->width()))
				{
					if(z >= zbuffer->getValue(row, column))
					{
						zbuffer->setValue(row, column, z);
						image->setPixel(column, row, tempColor.rgb());
					}
				}
			}
	}
}


