/***************************************************************************
 *   Copyright (C) 2007 by Lisandro Damián Nicanor Pérez Meyer             *
 *   perezmeyer@gmail.com                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "matrix4x4.h"
#include <QString>
#include <QChar>
#include <QDebug>

#include "point4f.h"

Matrix4X4::Matrix4X4()
{
	matrix[0][0] = 1.0;
	matrix[0][1] = 0.0;
	matrix[0][2] = 0.0;
	matrix[0][3] = 0.0;
	matrix[1][0] = 0.0;
	matrix[1][1] = 1.0;
	matrix[1][2] = 0.0;
	matrix[1][3] = 0.0;
	matrix[2][0] = 0.0;
	matrix[2][1] = 0.0;
	matrix[2][2] = 1.0;
	matrix[2][3] = 0.0;
	matrix[3][0] = 0.0;
	matrix[3][1] = 0.0;
	matrix[3][2] = 0.0;
	matrix[3][3] = 1.0;
}

void Matrix4X4::setValue(int i, int j, double value)
{
	if((i < dimension) && (j < dimension))
		matrix[i][j] = value;
}

Matrix4X4 Matrix4X4::multiplyToTheRight(const Matrix4X4 & rightMatrix) const
{
	double temp[dimension][dimension];
	Matrix4X4 result;

	for(int i = 0; i < dimension; i++)
	{
		temp[i][0] = matrix[i][0] * rightMatrix.getComponent(0, 0) +
		             matrix[i][1] * rightMatrix.getComponent(1, 0) +
		             matrix[i][2] * rightMatrix.getComponent(2, 0) +
		             matrix[i][3] * rightMatrix.getComponent(3, 0);
		temp[i][1] = matrix[i][0] * rightMatrix.getComponent(0, 1) +
		             matrix[i][1] * rightMatrix.getComponent(1, 1) +
		             matrix[i][2] * rightMatrix.getComponent(2, 1) +
		             matrix[i][3] * rightMatrix.getComponent(3, 1);
		temp[i][2] = matrix[i][0] * rightMatrix.getComponent(0, 2) +
		             matrix[i][1] * rightMatrix.getComponent(1, 2) +
		             matrix[i][2] * rightMatrix.getComponent(2, 2) +
		             matrix[i][3] * rightMatrix.getComponent(3, 2);
		temp[i][3] = matrix[i][0] * rightMatrix.getComponent(0, 3) +
		             matrix[i][1] * rightMatrix.getComponent(1, 3) +
		             matrix[i][2] * rightMatrix.getComponent(2, 3) +
		             matrix[i][3] * rightMatrix.getComponent(3, 3);
	}

	for(int i=0; i < dimension; i++)
		for(int j=0; j < dimension; j++)
			result.setValue(i, j, temp[i][j]);
	return result;
}

Point4F Matrix4X4::multiplyPoint(Point4F point)
{
	Point4F temp;
	temp.rx() = matrix[0][0] * point.x() + matrix[0][1] * point.y() +
	            matrix[0][2] * point.z() + matrix[0][3] * point.w();
	temp.ry() = matrix[1][0] * point.x() + matrix[1][1] * point.y() +
	            matrix[1][2] * point.z() + matrix[1][3] * point.w();
	temp.rz() = matrix[2][0] * point.x() + matrix[2][1] * point.y() +
	            matrix[2][2] * point.z() + matrix[2][3] * point.w();
	temp.rw() = matrix[3][0] * point.x() + matrix[3][1] * point.y() +
	            matrix[3][2] * point.z() + matrix[3][3] * point.w();

	return Point4F(temp);
}

double Matrix4X4::getComponent(const int & row, const int & column) const
{
	return matrix[row][column];
}

QString Matrix4X4::getMatrixString()
{
	QString str1, str2, str3, str4;
	str1 = QString("\n%1 %2 %3 %4\n").arg(matrix[0][0],5,'g',3,QChar(' '))
	                           .arg(matrix[0][1],5,'g',3,QChar(' '))
	                           .arg(matrix[0][2],5,'g',3,QChar(' '))
	                           .arg(matrix[0][3],5,'g',3,QChar(' '));
	str2 = QString("%1 %2 %3 %4\n").arg(matrix[1][0],5,'g',3,QChar(' '))
	                           .arg(matrix[1][1],5,'g',3,QChar(' '))
	                           .arg(matrix[1][2],5,'g',3,QChar(' '))
	                           .arg(matrix[1][3],5,'g',3,QChar(' '));
	str3 = QString("%1 %2 %3 %4\n").arg(matrix[2][0],5,'g',3,QChar(' '))
	                         .arg(matrix[2][1],5,'g',3,QChar(' '))
	                         .arg(matrix[2][2],5,'g',3,QChar(' '))
	                         .arg(matrix[2][3],5,'g',3,QChar(' '));
	str4 = QString("%1 %2 %3 %4\n").arg(matrix[3][0],5,'g',3,QChar(' '))
	                         .arg(matrix[3][1],5,'g',3,QChar(' '))
	                         .arg(matrix[3][2],5,'g',3,QChar(' '))
	                         .arg(matrix[3][3],5,'g',3,QChar(' '));

	return str1 + str2 + str3 + str4;
}

Matrix4X4 & Matrix4X4::operator=(Matrix4X4 & rightMatrix)
{
	if(this == &rightMatrix)
		return *this;

	for(int i=0; i < dimension; i++)
		for(int j=0; j < dimension; j++)
			matrix[i][j] = rightMatrix.getComponent(i, j);
	return *this;
}


