/***************************************************************************
 *   Copyright (C) 2007 by Lisandro Damián Nicanor Pérez Meyer             *
 *   perezmeyer@gmail.com                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef MATRIX4X4
#define MATRIX4X4

class QString;
class Point4F;

/// This class defines a 3 X 3 matrix
/**
	This class represents a 3 X 3 matrix that allows different operations on it.
*/

class Matrix4X4
{
public:
	/// Initializes the matrix as a zero matrix.
	Matrix4X4();

	/// Sets the value of the [i][j] component of the matrix
	void setValue(int i, int j, double value);

	/// Multiply to the right
	/**
		Multiplies the current matrix to the right by the rightMatrix.
	*/
	Matrix4X4  multiplyToTheRight(const Matrix4X4 & rightMatrix) const;

	/// Multiply a point
	/**
		Multiplies a point by the matrix and returns a point
	*/
	Point4F multiplyPoint(Point4F point);

	/// Get the row, column component of the matrix
	double getComponent(const int & row, const int & column) const;

	/// Returns a QString with the contents of the matrix
	QString getMatrixString();

	/// Operator =
	Matrix4X4 & operator=(Matrix4X4 & rightMatrix);

private:
	static const int dimension = 4;
	double matrix[dimension][dimension];
};

#endif // Matrix4X4

