/***************************************************************************
 *   Copyright (C) 2007 by Lisandro Damián Nicanor Pérez Meyer             *
 *   perezmeyer@gmail.com                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "practico5.h"
#include <QVariant>
#include <QDebug>

Practico5::Practico5(QWidget * parent) : QMainWindow(parent)
{
	// We set the main window
	mainWindow.setupUi(this);

	/*
		Fill the elements combo box. Note that the order must be the same as the
		declared in the header file.
	*/
	mainWindow.elementsComboBox->addItem(tr("Cube"));

	// Set the counters to -1
	cubeCount = -1;

	// Connect the perspective proyection checkbox
	connect(mainWindow.perspectiveProyectionCheckBox, SIGNAL(stateChanged(int)),
	        mainWindow.scene, SLOT(perspectiveProyection(int)));

	/*
		Connect the addPushButton's clicked() signal to our
		addFigureToScene() slot.
	*/
	connect(mainWindow.addPushButton, SIGNAL(clicked()),
	        this, SLOT(addFigureToScene()));

	/*
		Connect the mainWindow's figureListWidget rowChanged() signal to the
		mainWindow's scene figureSelected slot.
		This will trigger an update of the sliders.
	*/
	connect(mainWindow.figureListWidget, SIGNAL(currentRowChanged(int)),
	        mainWindow.scene, SLOT(figureSelected(int)));

	/*
		Connect the signal that sends the sliders parameters to the slot that
		updates them.
	*/
	connect(mainWindow.scene, SIGNAL(changeParameters(double, double,
	                                                  double, double,
	                                                  double, double,
	                                                  double)),
	        this, SLOT(resetSliders(double, double, double, double, double,
	                                double, double)));

	// Connect the sliders to the objects parameters
	connect(mainWindow.xDisplacementHorizontalSlider, SIGNAL(valueChanged(int)),
	        mainWindow.scene, SLOT(changeXDisplacement(int)));
	connect(mainWindow.yDisplacementHorizontalSlider, SIGNAL(valueChanged(int)),
	        mainWindow.scene, SLOT(changeYDisplacement(int)));
	connect(mainWindow.zDisplacementHorizontalSlider, SIGNAL(valueChanged(int)),
	        mainWindow.scene, SLOT(changeZDisplacement(int)));

	connect(mainWindow.xRotationHorizontalSlider, SIGNAL(valueChanged(int)),
	        mainWindow.scene, SLOT(changeAngleOfRotationX(int)));
	connect(mainWindow.yRotationHorizontalSlider, SIGNAL(valueChanged(int)),
	        mainWindow.scene, SLOT(changeAngleOfRotationY(int)));
	connect(mainWindow.zRotationHorizontalSlider, SIGNAL(valueChanged(int)),
	        mainWindow.scene, SLOT(changeAngleOfRotationZ(int)));

	connect(mainWindow.xScaleHorizontalSlider, SIGNAL(valueChanged(int)),
	        mainWindow.scene, SLOT(changeXScale(int)));
	connect(mainWindow.yScaleHorizontalSlider, SIGNAL(valueChanged(int)),
	        mainWindow.scene, SLOT(changeYScale(int)));
	connect(mainWindow.zScaleHorizontalSlider, SIGNAL(valueChanged(int)),
	        mainWindow.scene, SLOT(changeZScale(int)));

	// Connect the sliders to the scene parameters
	connect(mainWindow.xSceneDisplacementHorizontalSlider, SIGNAL(valueChanged(int)),
	        mainWindow.scene, SLOT(changeSceneXDisplacement(int)));
	connect(mainWindow.ySceneDisplacementHorizontalSlider, SIGNAL(valueChanged(int)),
	        mainWindow.scene, SLOT(changeSceneYDisplacement(int)));
	connect(mainWindow.zSceneDisplacementHorizontalSlider, SIGNAL(valueChanged(int)),
	        mainWindow.scene, SLOT(changeSceneZDisplacement(int)));

	connect(mainWindow.xSceneRotationHorizontalSlider, SIGNAL(valueChanged(int)),
	        mainWindow.scene, SLOT(changeSceneAngleOfRotationX(int)));
	connect(mainWindow.ySceneRotationHorizontalSlider, SIGNAL(valueChanged(int)),
	        mainWindow.scene, SLOT(changeSceneAngleOfRotationY(int)));
	connect(mainWindow.zSceneRotationHorizontalSlider, SIGNAL(valueChanged(int)),
	        mainWindow.scene, SLOT(changeSceneAngleOfRotationZ(int)));

	connect(mainWindow.xSceneScaleHorizontalSlider, SIGNAL(valueChanged(int)),
	        mainWindow.scene, SLOT(changeSceneXScale(int)));
	connect(mainWindow.ySceneScaleHorizontalSlider, SIGNAL(valueChanged(int)),
	        mainWindow.scene, SLOT(changeSceneYScale(int)));
	connect(mainWindow.zSceneScaleHorizontalSlider, SIGNAL(valueChanged(int)),
	        mainWindow.scene, SLOT(changeSceneZScale(int)));
}

void Practico5::addFigureToScene()
{
	// We must check what kind of figure the user wanted to add
	int element = mainWindow.elementsComboBox->currentIndex();

	if(element == cube)
	{
		mainWindow.scene->addCube();

		cubeCount++;
		mainWindow.figureListWidget->addItem(tr("Cube") +
		                                     QString(" - %1").arg(cubeCount));
	}
	else
	{
		qDebug() << "element does not match with any figure";
		return ;
	}

	mainWindow.scene->figureSelected(mainWindow.figureListWidget->count() - 1);

	// Now we load the predetermined values
	mainWindow.xDisplacementHorizontalSlider->setValue(0);
	mainWindow.yDisplacementHorizontalSlider->setValue(0);
	mainWindow.zDisplacementHorizontalSlider->setValue(0);
	mainWindow.xRotationHorizontalSlider->setValue(0);
	mainWindow.yRotationHorizontalSlider->setValue(0);
	mainWindow.zRotationHorizontalSlider->setValue(0);
	mainWindow.xScaleHorizontalSlider->setValue(50);
	mainWindow.yScaleHorizontalSlider->setValue(50);
	mainWindow.zScaleHorizontalSlider->setValue(50);
}

void Practico5::resetSliders(double xDisplacement, double yDisplacement,
                             double rotationAngleX, double rotationAngleY,
                             double rotationAngleZ, double xScale,
                             double yScale)
{
// 	qDebug() << "Sliders reseted" << (int)xDisplacement << xDisplacement;
// 	mainWindow.xDisplacementHorizontalSlider->setValue((int)xDisplacement);
// 	mainWindow.yDisplacementHorizontalSlider->setValue((int)yDisplacement);
// 	mainWindow.rotationHorizontalSlider->setValue((int)rotationAngle);
// 	mainWindow.xScaleHorizontalSlider->setValue((int)xScale);
// 	mainWindow.yScaleHorizontalSlider->setValue((int)yScale);
// 	qDebug() << "Sliders reseted" << (int)xDisplacement << xDisplacement;
}

