/***************************************************************************
 *   Copyright (C) 2007 by Lisandro Damián Nicanor Pérez Meyer             *
 *   perezmeyer@gmail.com                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "scanlinetable.h"
#include <QtAlgorithms>
#include <float.h>
#include <QDebug>

ScanLineTable::ScanLineTable(int imageHeight)
{
	offset = imageHeight/2;

	for(int i = 0; i< imageHeight; i++)
	{
		struct scanLineTableItem * item = new struct scanLineTableItem;
		item->minimumXCoordinate = DBL_MAX;
		item->maximumXCoordinate = -1 * DBL_MAX;
		item->minimumZCoordinate = DBL_MAX;
		item->maximumZCoordinate = -1 * DBL_MAX;
		item->draw = false;
		item->minColor = QColor("black");
		item->maxColor = QColor("black");
		table.append(item);
	}
}

ScanLineTable::~ScanLineTable()
{
	qDeleteAll(table.begin(), table.end());
	table.clear();
}

void ScanLineTable::addItem(double xCoordinate, int yCoordinate,
                            double zCoordinate, QColor color)
{
	if(yCoordinate < 0 || yCoordinate >= 2 * offset)
	{
// 		qDebug() << "yCoordinate is out of bonds. yCoordinate = " << yCoordinate << 2 * offset;
		return;
	}

	if(xCoordinate > table.at(yCoordinate)->maximumXCoordinate)
	{
		table[yCoordinate]->maximumXCoordinate = xCoordinate;
		table[yCoordinate]->maximumZCoordinate = zCoordinate;
		table[yCoordinate]->maxColor = color;
	}

	if(xCoordinate < table.at(yCoordinate)->minimumXCoordinate)
	{
		table[yCoordinate]->minimumXCoordinate = xCoordinate;
		table[yCoordinate]->minimumZCoordinate = zCoordinate;
		table[yCoordinate]->minColor = color;
	}

	table[yCoordinate]->draw = true;
}

void ScanLineTable::getXMinMax(int y, double & minX, double & minZ, QColor & minColor,
                               double & maxX, double & maxZ, QColor & maxColor)
{
	if(table[y]->draw)
	{
		minX = table[y]->minimumXCoordinate;
		maxX = table[y]->maximumXCoordinate;
		minZ = table[y]->minimumZCoordinate;
		maxZ = table[y]->maximumZCoordinate;
		minColor = table[y]->minColor;
		maxColor = table[y]->maxColor;
	}
	else
	{
		minX = 0;
		maxX = 0;
	}
}

int ScanLineTable::getSize()
{
	return 2 * offset;
}




